<?php

namespace Bydedi\FileUpload;

use Bydedi\FileUpload\Exception\ExtensionNotFoundException;
use Symfony\Component\HttpFoundation\File\Exception\FileException;
use Symfony\Component\HttpFoundation\File\UploadedFile;

/**
 * Class File
 * @package Bydedi\FileUpload
 */
class File
{

    /**
     * @var array
     */
    private static $types = null;
    /**
     * @var array
     */
    protected $extensions;
    /**
     * The max size of the file in bytes
     * @var string
     */
    protected $size;
    /**
     * @var string
     */
    protected $uploadDir;

    /**
     * File constructor.
     */
    public function __construct()
    {
        $this->extensions = array();

        if (is_null(self::$types)) {
            self::initType();
        }
    }

    /**
     * @param $ext
     * @return bool
     */
    public function isExtensionValid($ext)
    {
        if (empty($this->extensions)) {
            trigger_error(
                'Utilisation de la classe File sans restriction sur l\'extension du fichier',
                E_USER_WARNING
            );
            return true;
        }

        return in_array($ext, $this->extensions);
    }

    /**
     * @param $size
     * @return bool
     */
    public function isSizeValid($size)
    {
        if (empty($this->size)) {
            trigger_error(
                'Utilisation de la classe File sans restriction sur la taille du fichier',
                E_USER_WARNING
            );
            return true;
        }

        return ($size <= $this->size);
    }

    /**
     * @param UploadedFile $file
     * @return array|\Symfony\Component\HttpFoundation\File\File
     */
    public function moveFile(UploadedFile $file)
    {
        $this->createFolderIfNotExist();
        $name = date('YmdHis') . '_' . $this->parseFileName($file->getClientOriginalName());

        if (file_exists($this->uploadDir . '/' . $name)) {
            $name = rand(0, 9) . '_' . $name;
        }

        try {
            $newFile = $file->move($this->uploadDir, $name);
        } catch (FileException $e) {
            return array(
                'error' => $e->getMessage()
            );
        }

        return $newFile;
    }

    /**
     *
     */
    private static function initType()
    {
        // https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Complete_list_of_MIME_types
        self::$types = array();
        self::$types['.csv'] = 'text/csv';
        self::$types['.pdf'] = 'application/pdf';
        self::$types['.rtf'] = 'application/rtf';
        self::$types['.gif'] = 'image/gif';
        self::$types['.jpeg'] = 'image/jpeg';
        self::$types['.jpg'] = 'image/jpeg';
        self::$types['.png'] = 'image/png';
        self::$types['.txt'] = 'text/plain';
        self::$types['.doc'] = 'application/msword';
        self::$types['.docx'] = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document';
        self::$types['.xls'] = 'application/vnd.ms-excel';
        self::$types['.xlsx'] = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
        self::$types['.odt'] = 'application/vnd.oasis.opendocument.text';
        self::$types['.ods'] = 'application/vnd.oasis.opendocument.spreadsheet';
    }

    /**
     * @param $text
     * @return mixed|string
     */
    private function parseFileName($text)
    {
        $accents = "ÀÁÂÃÄÅàáâãäåÒÓÔÕÖØòóôõöøÈÉÊËèéêëÇçÌÍÎÏìíîïÙÚÛÜùúûüÿÑñ";
        $ssaccents = "AAAAAAaaaaaaOOOOOOooooooEEEEeeeeCcIIIIiiiiUUUUuuuuyNn";
        $text = strtr($text, $accents, $ssaccents);
        $input = array(' - ', ' ', '?', '!', ',', ':', "'", '&', '(', ')', '/', '"', '+', '*', '%', '?');
        $output = array('-', '-', '', '', '', '', '-', 'et', '', '', '', '', '', '', '', '');
        $text = str_replace($input, $output, $text);
        $text = strtolower($text);

        return $text;
    }

    /**
     * @param UploadedFile $file
     * @return array
     */
    public function checkFile(UploadedFile $file)
    {
        $errors = array();

        if ($file->getError() != UPLOAD_ERR_OK) {
            $errors['form_upload_error'] = 'Une erreur s\'est produite lors de l\'envoi de votre fichier';
        }
        if (!$this->isExtensionValid($file->getClientMimeType())) {
            $errors['form_upload_error_extension'] =
                "Le fichier n'a pas une extension valide. Extensions autorisées : "
                . implode(", ", array_keys($this->extensions))
            ;
        }
        if (!$this->isSizeValid($file->getSize())) {
            $errors['form_upload_error_taille'] = 'Le fichier a une taille trop volumineuse';
        }

        return $errors;
    }

    /**
     * @param UploadedFile $file
     * @return array|\Symfony\Component\HttpFoundation\File\File
     */
    public function uploadFile(UploadedFile $file)
    {
        $errors = $this->checkFile($file);

        if (!empty($errors)) {
            return $errors;
        }

        return $this->moveFile($file);
    }

    /**
     * @return bool
     */
    public function canUpload()
    {
        $this->createFolderIfNotExist();
        return is_writable($this->uploadDir);
    }

    /**
     * Crée le dossier de destination s'il n'existe pas
     */
    private function createFolderIfNotExist()
    {
        if (!is_dir($this->uploadDir)) {
            mkdir($this->uploadDir, 0775, true);
        }
    }

    /**
     * @param $file
     * @return bool
     */
    public function removeFile($file)
    {
        if (!file_exists($file)) {
            return false;
        }

        return unlink($file);
    }

    /**
     * @return string
     */
    public function getUploadDir()
    {
        return $this->uploadDir;
    }

    /**
     * @param $uploadDir
     * @return $this
     */
    public function setUploadDir($uploadDir)
    {
        $this->uploadDir = $uploadDir;
        return $this;
    }

    /**
     * @return string
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * @param string $size
     * @return $this
     */
    public function setSize($size)
    {
        $this->size = $this->convertToByte($size);
        return $this;
    }

    /**
     * @return array
     */
    public function getExtensions()
    {
        return $this->extensions;
    }

    /**
     * @param $extensions
     * @return $this
     */
    public function setExtensions($extensions)
    {
        $this->extensions = $extensions;
        return $this;
    }

    /**
     * @param $extension
     * @return $this
     * @throws \Exception
     */
    public function addExtension($extension)
    {
        if (!isset(self::$types[$extension])) {
            throw new ExtensionNotFoundException("", 0, null, $extension);
        }

        $this->extensions[$extension] = self::$types[$extension];
        return $this;
    }

    /**
     * convert a size "human readable" to bytes
     * 2mb => 2097152
     *
     * @param $value
     * @return null|string|string[]
     */
    private function convertToByte($value)
    {
        return preg_replace_callback('/^\s*(\d+)\s*(?:([kmgt]?)b?)?\s*$/i', function ($parts) {
            switch (strtolower($parts[2])) {
                case 't':
                    $parts[1] *= 1024;
                // no break
                case 'g':
                    $parts[1] *= 1024;
                // no break
                case 'm':
                    $parts[1] *= 1024;
                // no break
                case 'k':
                    $parts[1] *= 1024;
            }
            return $parts[1];
        }, $value);
    }
}
